// All of the Node.js APIs are available in the preload process.
// It has the same sandbox as a Chrome extension.
import { contextBridge, IpcRenderer } from 'electron';
import { Job } from './src/app/job';
import { AppAction, Strudelapp, StrudelappInstance } from './src/app/strudelapp';
const ipcRenderer: IpcRenderer = require('electron').ipcRenderer;
contextBridge.exposeInMainWorld(
  'electronApi',
  {
    addCert: (data: any, fn: () => void): void  => { addCert(data, fn) },
    register_oauth2_redirect: (fn: (arg0: string) => void): void => {register_oauth2_redirect(fn)},
    launch: (job: Job, appinst: StrudelappInstance, action: AppAction, fn: (args) => void): void => { launch(job, appinst, action, fn)},
    runRemoteCommand: (data: any, fn: (args) => void): void => { runRemoteCommand(data, fn)},
    createTunnel: (data: any, fn: (args) => void): void => { createTunnel(data, fn)},
    sshAgent: (data: any, fn: (args) => void): void => { sshAgent(data, fn)}

  }
)

function addCert(data: any, fn: any): void {
  ipcRenderer.once('addCertResponse', (event, ...args) => fn() )
  ipcRenderer.send('addCert', (data));
}

function register_oauth2_redirect(fn: (arg0: string) => void ): void {
  ipcRenderer.once('oauth2-redirect',(event, url) => fn(url));
}

function launch(job: Job, appinst: StrudelappInstance, action: AppAction, fn: any): void {
  ipcRenderer.once('launchResponse', (event, ...args) => fn(args) )
  ipcRenderer.send('launch', ([job, appinst, action]));
}

function runRemoteCommand(data: any, fn: (args)=>void ): void {
  if (data.hasOwnProperty('tag')) {
    ipcRenderer.once(data['tag'], (event, ...args) => fn(args) )
  }
  else {
    ipcRenderer.once('runRemoteCommandResponse'+data['cmd'], (event, ...args) => fn(args) )
  }
  ipcRenderer.send('runRemoteCommand', (data));
}

function createTunnel(data: any, fn: (args)=>void): void {
  ipcRenderer.once('createTunnelResponse', (event, ...args) => fn(args) )
  ipcRenderer.send('createTunnel', (data));
}
function sshAgent(data: any, fn: (args)=>void): void {
  ipcRenderer.once('sshAgentResponse', (event, ...args) => fn(args) )
  ipcRenderer.send('sshAgent', (data));
}